
/* sparse matrix routines, by William Menke, November 2001 */
/* a simple package that implements efficient storage of   */
/* symmetric and asymmetric sparse matrices, and common    */
/* operations, such as multiplication, applied to them     */

/* you declare a pointer to a n by m sparse matrix, and this  */
/* routine initializes it to a matrix of zeros.  Use sym=0    */
/* for a asymmetric matrix, sym=1 for a square symmetric one  */
struct sparse_matrix *new_sparse_matrix(int n, int m, int sym )
{
	int i;
	struct sparse_matrix *matrix;
	struct sparse_elem **row, **col;

	if( (n<1) || (m<1) || ((n!=m)&&sym) ) return(NULL);

	matrix = (struct sparse_matrix *) malloc(sizeof(struct sparse_matrix));
	if( matrix==NULL ) return(NULL);
	matrix->n=n; matrix->m=m; matrix->count=0; matrix->sym=sym;

	row = (struct sparse_elem **) malloc(n*sizeof(struct sparse_elem *));
	if( row==NULL ) {free(matrix); return(NULL);}
	else { matrix->row=row; for( i=0; i<n; i++ ) row[i]=NULL; }

	col = (struct sparse_elem **) malloc(m*sizeof(struct sparse_elem *));
	if( col==NULL ) {free(matrix); free(row); return(NULL);}
	else { matrix->col=col; for( i=0; i<m; i++ ) col[i]=NULL; }

	return(matrix);
	}

/* copy a sparse matrix */
struct sparse_matrix *copy_sparse_matrix(struct sparse_matrix *A)
{
	int i;
	struct sparse_matrix *matrix;
	struct sparse_elem **row, **col, *e;

	if( A==NULL ) return(NULL);

	matrix = (struct sparse_matrix *) malloc(sizeof(struct sparse_matrix));
	if( matrix==NULL ) return(NULL);
	matrix->n=A->n; matrix->m=A->m; matrix->count=A->count; matrix->sym=A->sym;

	row = (struct sparse_elem **) malloc(A->n*sizeof(struct sparse_elem *));
	if( row==NULL ) {free(matrix); return(NULL);}
	else { matrix->row=row; for( i=0; i<A->n; i++ ) row[i]=NULL; }

	col = (struct sparse_elem **) malloc(A->m*sizeof(struct sparse_elem *));
	if( col==NULL ) {free(matrix); free(row); return(NULL);}
	else { matrix->col=col; for( i=0; i<A->m; i++ ) col[i]=NULL; }

	for( i=0; i<A->n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			if( !set_sparse_value(matrix,e->i,e->j,e->x) ) return(NULL);
			e=e->no;
			}
		}

	return(matrix);
	}

/* transpose a sparse matrix in place (ie A is changed to A-transpose) */
int sparse_transpose(struct sparse_matrix *A)
{
	int i, j;
	//struct sparse_matrix *matrix; //unused
	struct sparse_elem **row, *e, *next_e; //, **p; //unused

	if( A==NULL ) return(0);
	if( A->sym ) return(1);

	for( i=0; i<A->n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			next_e=e->no; e->no=e->nd; e->nd=next_e;
			j=e->j; e->j=e->i; e->i=j;
			e=next_e;
			}
		}
	row=A->row;
	A->row=A->col;
	A->col=row;

	j=A->n; A->n=A->m; A->m=j;

	return(1);
	}


/* accesses the dimensions n, m and number of non-zero elements */
int sparse_dim( struct sparse_matrix *A, int *n, int *m, int *count )
{
	if( A==NULL ) return(0);
	else {
		*n = A->n; *m=A->m; *count=A->count;
		return(1);
		}
	}

/* is the sparse matrix A symmetric ? */
int sparse_issym( struct sparse_matrix *A )
{
	if( A==NULL ) return(0);
	else {
		return(A->sym);
		}
	}

/* return count of non-zero elements*/
int sparse_count( struct sparse_matrix *A )
{
	if( A==NULL ) return(0);
	else return(A->count);
	}

/* delete a sparse matrix */
int free_sparse_matrix( struct sparse_matrix *m )
{
	int i;
	struct sparse_elem *e, *ep;
	for( i=0; i<m->n; i++ ) {
		e=m->row[0];
		while( e!=NULL ) {
			ep=e->no;
			free(e);
			e=ep;
			}
		}
	return(1);
	}

/* gets the value of element m[i][j] */
double get_sparse_value( struct sparse_matrix *m, int i, int j)
{
	struct sparse_elem *e;
	int k;

	if( m->sym && (i>j) ) { k=i; i=j; j=k; };

	if( m->n < m->m ) { 
		e=m->row[i];
		while( e!=NULL ) {
			if( e->j == j ) return(e->x); else e=e->no;
			}
		}
	else {
		e=m->col[j];
		while( e!=NULL ) {
			if( e->i == i ) return(e->x); else e=e->nd;
			}
		}
		
	return((double)0.0);
	}

/* sets the value of m[i][j]=x. */
int set_sparse_value( struct sparse_matrix *m, int i, int j, double x)
{
	struct sparse_elem *r, *rl, *c, *cl, *e;
	int inm, k;

	if( (i<0) || (j<0) || (i>=m->n) || (j>=m->m) ) return(0);

	if( m->sym && (i>j) ) { k=i; i=j; j=k; }

	if( x==0.0 ) {
		c=m->row[i];
		cl=NULL;
		while( c!=NULL ) {
			if( c->j == j ) break;
			cl=c;
			c=c->no;
			}
		/* case 1: this element not in matrix, so do nothing */
		if( c == NULL ) {
			return(1);
			}

		/* case 2: this element in matrix, so remove it */
		if( cl != NULL ) cl->no = c->no;
		if( m->row[i] == c ) m->row[i]=c->no;
		r=m->col[j];
		rl=NULL;
		while( r!=NULL ) {
			if( r == c ) break;
			rl=r;
			r=r->nd;
			}
		if( r != c ) {
			fprintf(stderr,"sparse matrix bug: nomatch\n");
			exit(255);
			}

		if( rl != NULL ) rl->nd = r->nd;
		if( m->col[j] == r ) m->col[j]=r->nd;
		free(c);
		m->count--;
		return(1);
		}

	else {
		inm=0;
		c=NULL; cl=NULL;
		if( m->row[i]!=NULL ) {
			c=m->row[i];
			cl=NULL;
			while( c!=NULL ) {
				if( c->j==j ) { inm=1; break; }
				if( c->j>j ) break;
				cl=c;
				c=c->no;
				}
			}

		if( inm ) { /* case 3: element is the the matrix and needs to be reset */
			c->x = x;
			return(1);
			}

		else { /* case 4: element is no in matrix and must be inserted */

			r=m->col[j];
			rl=NULL;
			while( r!=NULL ) {
				if( r->i>i ) break;
				rl=r;
				r=r->nd;
				}

			e = (struct sparse_elem *) malloc( sizeof(struct sparse_elem) );
			if( e==NULL ) return(0);
			m->count++;
			e->i=i; e->j=j; e->x=x; e->nd=NULL; e->no=NULL;
			if( rl!=NULL ) rl->nd=e; e->nd=r;
			if( cl!=NULL ) cl->no=e; e->no=c;
			if( m->row[i] == NULL ) m->row[i]=e;
			else if( m->row[i] == c ) m->row[i]=e;
			if( m->col[j] == NULL ) m->col[j]=e;
			else if( m->col[j] == r ) m->col[j]=e;
			return(1);
			}
		}
	}

/* add a vector vec to column j of sparse matrix A   */
/* zero-values of vec are ignored.  The element      */
/* is not removed if its new value is exactly zero   */
/* WARNING: Will not work on symmetric matrices!     */
int sparse_add_to_col( struct sparse_matrix *A, int j, double *vec )
{
	int i, n, m;
	int *isnew;
	struct sparse_elem *r, *rl, **p, *e, *c, *cl;
	// int inm; //unused

	if( A==NULL ) return(0);
	if( A->sym ) return(0);
	n=A->n; m=A->m;

	/* create column pointer and new-flag */
	p=(struct sparse_elem **)malloc(n*sizeof(struct sparse_elem *));
	if( p==NULL ) return(0);
	isnew=(int*)malloc(n*sizeof(int));
	if( isnew==NULL ) {free(p); return(0); }

	/* initial column pointer and new-flag */
	for( i=0; i<n; i++ ) {
		p[i]=NULL;
		isnew[i]=0;
		}

	/* copy existing elements of column into p */
	e=A->col[j];
	while( e!=NULL ) {
		p[e->i]=e;
		e=e->nd;
		}

	/* add vec to elements of column, creating new ones as necessary */
	for( i=0; i<n; i++ ) if( vec[i]!=0.0 ) {
		if( p[i]==NULL ) {
			p[i]=(struct sparse_elem*)malloc(sizeof(struct sparse_elem));
			if(p[i]==NULL) {free(p);free(isnew);return(0);}
			A->count++;
			p[i]->i=i; p[i]->j=j; p[i]->x=vec[i]; p[i]->nd=NULL; p[i]->no=NULL;
			isnew[i]=1;
			}
		else {
			p[i]->x+=vec[i];
			}
		}

	/* relink the next-down pointers and the start-of-column pointer */
	r=NULL; rl=NULL; A->col[j]=NULL;
	for( i=0; i<n; i++ ) if( p[i]!=NULL ) {
		r=p[i];
		r->nd=NULL;
		if( rl==NULL ) A->col[j]=r; else rl->nd=r;
		rl=r;
		}

	/* relink the next-over pointers and the start-of-row pointer */
	/* only need to process rows with new elements                */
	for( i=0; i<n; i++ ) if( isnew[i] ) {
		if( A->row[i]==NULL ) { /* this is the only element of the row */
			A->row[i]=p[i];
			p[i]->no=NULL;
			}
		else { 
			/* locate position: column j between cl and c */
			c=A->row[i]; cl=NULL;
			while( c!=NULL ) {
				if( c->j > j ) break;
				cl=c;
				c=c->no;
				}
			if( cl==NULL ) { /* no last-column, so thus is first element */
				p[i]->no=A->row[i];
				A->row[i]=p[i];
				}
			else {
				p[i]->no = cl->no;
				cl->no=p[i];
				}
			}
		}

	free(p); free(isnew);							
	return(1);
	}



/* b = A * a, with A sparse and a,b not sparse*/
int sm_times_v( struct sparse_matrix *A, double *a, double *b )
{
	int i, n, k, iscol;
	struct sparse_elem *c;
	double sum;

	n=A->n;
	if( A==NULL) {
		return(0);
		}
	else if( ! A->sym ) {
		for( i=0; i<n; i++ ) {
			sum=0.0;
			c=A->row[i];
			while( c!=NULL ) {
				sum += c->x * a[c->j];
				c = c->no;
				}
			b[i]=sum;
			}
		}
	else {
		for( i=0; i<n; i++ ) {
			iscol=1;
			c=A->col[i]; if(c==NULL){ c=A->row[i]; iscol=0; }
			sum=0.0;
			while( c!=NULL ) {
				if( iscol ) k=c->i; else k=c->j;
				sum += c->x * a[k];
				if( iscol ) {
					c=c->nd;
					if( c==NULL ) {
						iscol=0;
						c=A->row[i];
						if( c!=NULL ) if(c->i==c->j) c=c->no;
						}
					}
				else c=c->no;
				}
			b[i]=sum;
			}
		}

	return(1);
	}

/* b = A-transpose * a, with A sparse and a,b not sparse*/
int smt_times_v( struct sparse_matrix *A, double *a, double *b )
{
	int j, m;
	struct sparse_elem *r;
	double sum;

	if( a==NULL) return(0);
	else if( A->sym ) return( sm_times_v(A,a,b) );
	else {

		m=A->m;
		for( j=0; j<m; j++ ) {
			sum=0.0;
			r=A->col[j];
			while( r!=NULL ) {
				sum += r->x * a[r->i];
				r = r->nd;
				}
			b[j]=sum;
			}
		return(1);
		}

	}


/* C = A * B, with A,B sparse*/
struct sparse_matrix *sm_times_sm( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB;
	double sum;
	int iscolA, iscolB;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( mA != nB ) return(NULL);

	if( (!A->sym) && (!B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			eA=A->row[i]; eB=B->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j; kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no; else eB=eB->nd;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else if( (A->sym) && (!B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if( iscolA ) {
					    eA=eA->nd;
					    if( eA==NULL ) {
					        iscolA=0;
					        eA=A->row[i];
					        if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
					        }
					    }
					else eA=eA->no;
					}
				else eB=eB->nd;
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	else if( (!A->sym) && (B->sym) ) {

		C=new_sparse_matrix(nA,mB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			eA=A->row[i];
			iscolB=1;
			eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j;
				if(iscolB) kB=eB->i; else kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no;
				else {
					if( iscolB ) {
					    eB=eB->nd;
					    if( eB==NULL ) {
						iscolB=0;
						eB=B->row[j];
						if(eB!=NULL) if(eB->i==eB->j) eB=eB->no;
						}
					    }
					else eB=eB->no;
					}
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	else if( (A->sym) && (B->sym) ) {

		if( A!=B ) {
			C=new_sparse_matrix(nA,mB,0);
			if( C==NULL ) return(NULL);
			for( i=0; i<nA; i++ ) for( j=0; j<mB; j++ ) {
				sum=0.0;
				iscolA=1; iscolB=1;
				eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
				eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
				while( (eA!=NULL) && (eB!=NULL) ) {
					if(iscolA) kA=eA->i; else kA=eA->j;
					if(iscolB) kB=eB->i; else kB=eB->j;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) {
					     if(iscolA){
						eA=eA->nd;
						if( eA==NULL ) {
							iscolA=0;
							eA=A->row[i];
							if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
							}
						    }
						else eA=eA->no;
						}
					else {
					    if(iscolB){
						eB=eB->nd;
						if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB=eB->no;
							}
						    }
						else eB=eB->no;
						}
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		else {
			C=new_sparse_matrix(nA,mB,1);
			if( C==NULL ) return(NULL);

			for( i=0; i<nA; i++ ) for( j=i; j<mB; j++ ) {
				sum=0.0;
				iscolA=1; iscolB=1;
				eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
				eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
				while( (eA!=NULL) && (eB!=NULL) ) {
					if(iscolA) kA=eA->i; else kA=eA->j;
					if(iscolB) kB=eB->i; else kB=eB->j;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) {
					    if(iscolA){
						eA=eA->nd;
						     if( eA==NULL ) {
							iscolA=0;
							eA=A->row[i];
							if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
							}
						    }
						else eA=eA->no;
						}
					else {
						if(iscolB){
						    eB=eB->nd;
						    if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB = eB->no;
							}
						    }
						else eB=eB->no;
						}
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		}

	return(C);
	}

/* C = A-transpose * B-transpose, with A,B sparse*/
struct sparse_matrix *smt_times_smt( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( nA != mB ) return(NULL);

	if( A->sym && B->sym ) return( sm_times_sm(A,B) );
	else if( A->sym && !B->sym ) return( sm_times_smt(A,B) );
	else if( !A->sym && B->sym ) return( smt_times_sm(A,B) );

	C=new_sparse_matrix(mA,nB,0);
	if( C==NULL ) return(NULL);

	for( i=0; i<mA; i++ ) for( j=0; j<nB; j++ ) {
		sum=0.0;
		eA=A->col[i]; eB=B->row[j];
		while( (eA!=NULL) && (eB!=NULL) ) {
			kA=eA->i; kB=eB->j;
			if( kA==kB ) sum += eA->x * eB->x;
			if( kA <= kB ) eA=eA->nd; else eB=eB->no;
			}
		set_sparse_value(C,i,j,sum);
		}

	return(C);
	}



/* returns absolute max element of ATA (assuming it to be on main diagonal) */
double ATAabsmax( struct sparse_matrix *A)
{
	struct sparse_elem *eA;
	int i, j, mA, iscolA, nA;
	double sum, t, absmax;

	if( A==NULL ) return(0.0);
	nA=A->n; mA=A->m;

	if( A->sym ) {
		for( i=0; i<mA; i++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			while( eA!=NULL ) {
				sum += eA->x * eA->x;
				if( iscolA ) {
					eA=eA->nd;
					if( eA==NULL ) {
						iscolA=0;
					        eA=A->row[i];
					        if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
					        }
					else {
						eA=eA->no;
						}
					}
				}
			t = fabs(sum);
			if( i==0 ) {
				absmax = t;
				}
			else if ( t > absmax ) {
				absmax = t;
				}
			}
		}
	else {
		for( i=0; i<mA; i++ ) {
			sum=0.0; j=0;
			eA=A->col[i];
			while( eA!=NULL ) {
				sum += eA->x * eA->x;
				j++;
				eA=eA->nd;
				}
			t = fabs(sum);
			if( i==0 ) {
				absmax = t;
				}
			else if ( t > absmax ) {
				absmax = t;
				}
			}
		}

	return(absmax);
	}

/* C = A * B-transpose, with A,B sparse*/
struct sparse_matrix *sm_times_smt( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB, iscolA;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( mA != mB ) return(NULL);

	if( B->sym ) return( sm_times_sm(A,B) );
	else if( !A->sym ) {
		C=new_sparse_matrix(nA,nB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<nB; j++ ) {
			sum=0.0;
			eA=A->row[i]; eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->j; kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->no; else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else if (A==B) { 
		C=new_sparse_matrix(nA,nB,1);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=i; j<nB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if(iscolA){
					    eA=eA->nd;
					    if( eA==NULL ) {
						iscolA=0;
						eA=A->row[i];
						if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
						}
					    }
					else eA=eA->no;
					}
				else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}
	else { 
		C=new_sparse_matrix(nA,nB,0);
		if( C==NULL ) return(NULL);

		for( i=0; i<nA; i++ ) for( j=0; j<nB; j++ ) {
			sum=0.0;
			iscolA=1;
			eA=A->col[i]; if(eA==NULL) { eA=A->row[i]; iscolA=0; }
			eB=B->row[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				if(iscolA) kA=eA->i; else kA=eA->j;
				kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) {
					if(iscolA){
					    eA=eA->nd;
					    if( eA==NULL ) {
						iscolA=0;
						eA=A->row[i];
						if(eA!=NULL) if(eA->i==eA->j) eA=eA->no;
						}
					    }
					else eA=eA->no;
					}
				else eB=eB->no;
				}
			set_sparse_value(C,i,j,sum);
			}
		}

	return(C);
	}

/* C = A-transpose * B, with A,B sparse*/
struct sparse_matrix *smt_times_sm( struct sparse_matrix *A, struct sparse_matrix *B)
{
	struct sparse_matrix *C;
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, nA, mA, nB, mB, iscolB;
	double sum;

	if( (A==NULL) || (B==NULL) ) return(NULL);
	nA=A->n; mA=A->m;
	nB=B->n; mB=B->m;
	if( nA != nB ) return(NULL);

	if( A->sym ) return( sm_times_sm(A,B) );
	else if (!B->sym) {
		if( A==B ) {
			C=new_sparse_matrix(mA,mB,1);
			if( C==NULL ) return(NULL);
			for( i=0; i<mA; i++ ) for( j=i; j<mB; j++ ) {
				sum=0.0;
				eA=A->col[i]; eB=B->col[j];
				while( (eA!=NULL) && (eB!=NULL) ) {
					kA=eA->i; kB=eB->i;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		else {
			C=new_sparse_matrix(mA,mB,0);
			if( C==NULL ) return(NULL);
			for( i=0; i<mA; i++ ) for( j=0; j<mB; j++ ) {
				sum=0.0;
				eA=A->col[i]; eB=B->col[j];
				while( (eA!=NULL) && (eB!=NULL) ) {
					kA=eA->i; kB=eB->i;
					if( kA==kB ) sum += eA->x * eB->x;
					if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
					}
				set_sparse_value(C,i,j,sum);
				}
			}
		}

	else {
		C=new_sparse_matrix(mA,mB,0);
		if( C==NULL ) return(NULL);
		for( i=0; i<mA; i++ ) for( j=0; j<mB; j++ ) {
			sum=0.0;
			iscolB=1;
			eA=A->col[i];
			eB=B->col[j]; if(eB==NULL) { eB=B->row[j]; iscolB=0; }
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->i;
				if(iscolB) kB=eB->i; else kB=eB->j;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->nd;
				else {
					if(iscolB){
						eB=eB->nd;
						if( eB==NULL ) {
							iscolB=0;
							eB=B->row[j];
							if(eB!=NULL) if(eB->i==eB->j) eB = eB->no;
							}
						}
					else eB=eB->no;
					}
				}
			set_sparse_value(C,i,j,sum);
			}
		}


	return(C);
	}

/* largest absolute value of elements */
double sparse_absmax(struct sparse_matrix *A)
{
	double x, v;
	int i, n;
	struct sparse_elem *e;

	v=0.0;
	n=A->n;
	for( i=0; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			x=fabs(e->x);
			if( x>v ) v=x;
			e=e->no;
			}
		}

	return(v);
	}

/* A = A + xI where A is sparse and x is a constant */
int sparse_addtodiag(struct sparse_matrix *A, double x)
{
	struct sparse_elem *e;
	int i, n;

	if( A==NULL) return(0);

	n=A->n;
	for( i=0; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			if( e->i == e->j ) e->x += x;
			e=e->no;
			}
		}

	return(1);
	}

/* truncate rows off bottom of sparse matrix, A*/
/* Note: A must not be symmetric */
int sparse_truncaterows( struct sparse_matrix *A, int newn ) 
{
	int i, j, n, m;
	struct sparse_elem *e, *ec;

	if( A==NULL ) return(0);
	if( A->sym ) return(0);
	n=A->n; m=A->m;
	if(newn>n) return(0); else if(newn==n) return(1);

	/* adjust all column pointers to cut off last rows */
	for( j=0; j<m; j++ ) {
		e=A->col[j];
		while( e!=NULL ) {
			ec=e->nd;
			if( ec != NULL ) if( ec->i >= newn ) { e->nd=NULL; break; }
			e=ec;
			}
		}

	/* delete rows */
	for( i=newn; i<n; i++ ) {
		e=A->row[i];
		while( e!=NULL ) {
			ec = e;
			e = e->no;
			free(ec);
			}
		}

	A->n=newn;

	return(1);
	}
		
#define EPS (1.0e-14)

/*solves n by n system Ax=b for x by bicongugate gradient method
   A: sparse matrix (must be square)
   b: l.h.s. vector
   x: solution; initialize to guess (e.g. 0.0) before call
   itol: 1, 2, 3, 4 depending on choice of convergence test
   tol: tolerance
   itmax: maximum iterations
   iter: actual iterations
   err: error estimate
   */
int sparse_asolve( struct sparse_matrix *A, double *b, double *x, int transpose );
int sparse_atimes( struct sparse_matrix *A, double *x, double *r, int transpose );
double snorm( int n, double *sx, int itol );

int sparse_biconj( struct sparse_matrix *A, double *b, double *x, int itol, double tol,
	int itmax, int *iter, double *err, int *biec)
{
	int j, n;
	double ak, akden, bk, bkden, bknum, bnorm, dxnorm, xnorm, zm1norm, znorm, t;
	double *p=NULL, *pp=NULL, *r=NULL, *rr=NULL, *z=NULL, *zz=NULL;

	if( A==NULL ) {
		fprintf(stderr,"error: biconj: NULL Matrix\n");
		*biec=1;
		return(0);
		}

	if( A->n != A->m ) {
		fprintf(stderr,"error: biconj: non-square Matrix\n");
		*biec=2;
		return(0);
		}

	if( (itol<1) || (itol>4) ) {
		fprintf(stderr,"error: biconj: bad itol\n");
		*biec=3;
		return(0);
		}

	n=A->n;

	p=(double*)malloc((n+10)*sizeof(double));
	pp=(double*)malloc((n+10)*sizeof(double));
	r=(double*)malloc((n+10)*sizeof(double));
	rr=(double*)malloc((n+10)*sizeof(double));
	z=(double*)malloc((n+10)*sizeof(double));
	zz=(double*)malloc((n+10)*sizeof(double));
	if( (p==NULL) || (pp==NULL) || (r==NULL) || (rr==NULL)
		                    || (z==NULL) || (zz==NULL) ) {
		if( p!=NULL ) free(p);
		if( pp!=NULL ) free(pp);
		if( r!=NULL ) free(r);
		if( rr!=NULL ) free(rr);
		if( z!=NULL ) free(z);
		if( zz!=NULL ) free(zz);
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}

	*iter=0;
	sparse_atimes(A,x,r,0);
	for( j=0; j<n; j++ ) {
		r[j]=b[j]-r[j];
		rr[j]=r[j];
		}
	znorm=1.0;
	if( itol==1) {
		bnorm=snorm(n,b,itol);
		}
	else if( itol==2 ) {
		sparse_asolve(A,b,z,0);
		bnorm=snorm(n,z,itol);
		}
	else {
		sparse_asolve(A,b,z,0);
		bnorm=snorm(n,z,itol);
		sparse_asolve(A,r,z,0);
		znorm=snorm(n,z,itol);
		}

	sparse_asolve(A,r,z,0);

	while( (*iter) <= itmax ) {
		(*iter)++;
		zm1norm=znorm;
		sparse_asolve(A,rr,zz,1);
		bknum=0.0;
		for( j=0; j<n; j++ ) bknum+=z[j]*rr[j];
		if( (*iter)==1 ) {
			for( j=0; j<n; j++ ) {
				p[j]=z[j];
				pp[j]=zz[j];
				}
			}
		else {
			bk=bknum/bkden;
			for( j=0; j<n; j++ ) {
				p[j]=bk*p[j]+z[j];
				pp[j]=bk*pp[j]+zz[j];
				}
			}
		bkden=bknum;
		sparse_atimes(A,p,z,0);
		akden=0.0;
		for( j=0; j<n; j++ ) akden+=z[j]*pp[j];
		ak=bknum/akden;
		sparse_atimes(A,pp,zz,1);
		for( j=0; j<n; j++ ) {
			x[j]+=ak*p[j];
			r[j]-=ak*z[j];
			rr[j]-=ak*zz[j];
			}
		sparse_asolve(A,r,z,0);
		if( (itol==1) || (itol==2) ) {
			znorm=1.0;
			(*err)=snorm(n,r,itol)/bnorm;
			}
		else {
			znorm=snorm(n,z,itol);
			if( (t=fabs(zm1norm-znorm)) > (EPS*znorm) ) {
				dxnorm=fabs(ak)*snorm(n,p,itol);
				(*err)=znorm/t*dxnorm;
				}
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			xnorm=snorm(n,x,itol);
			if( (*err)<=(0.5*xnorm) ) (*err)/=xnorm;
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			}
		if( (*err)<=tol) break;
		}

	free(p); free(pp); free(r); free(rr); free(z); free(zz);
	if( (*err)<=tol) {*biec=0; return(1);}
	else {*biec=(-1); return(0);}
	}



	
/* internal routine, not intended as user callable */
double snorm( int n, double *sx, int itol )
{
	int i;
	double ans, t;
	if( itol<=3) {
		ans=0.0;
		for(i=0; i<n; i++ ) ans+=sx[i]*sx[i];
		ans=sqrt(ans);
		}
	else {
		ans=fabs(sx[0]);
		for(i=0; i<n; i++ ) if( (t=fabs(sx[i]))>ans) {
			ans=t;
			}
		}
	return(ans);
	}

/* internal routine, not intended as user callable */
int sparse_atimes( struct sparse_matrix *A, double *x, double *r, int transpose )
{
	// int i, j; //unused
	if( transpose ) smt_times_v( A, x, r ); else sm_times_v( A, x, r );
	return(1);
	}

/* internal routine, not intended as user callable */
int sparse_asolve( struct sparse_matrix *A, double *b, double *x, int transpose )
{
	int i, n;
	double t;
	n = A->n;
	for( i=0; i<n; i++ ) {
		t=get_sparse_value(A,i,i);
		if( t!=0.0 ) x[i]=b[i]/t;
		else x[i]=0.0;
		}
	return(1);
	}
/* test routine, prints matrix */
int dump_sparse_matrix(struct sparse_matrix *A, FILE *f)
{
	struct sparse_elem *e;
	int i, j, n, m;

	n=A->n; m=A->m;

	if(A->sym)
	fprintf(f,"Dump of symm %d by %d sparse martix with %d non-zero elements\n", n, m, A->count );
	else
	fprintf(f,"Dump of asym %d by %d sparse martix with %d non-zero elements\n", n, m, A->count );


	j=0;
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		while( e != NULL ) {
			j++;
			e=e->no;
			}
		}
	fprintf(f,"elements accessibe from row pointers: %d\n", j );

	i=0;
	for( j=0; j<m; j++ ) {
		e= A->col[j];
		while( e != NULL ) {
			i++;
			e=e->nd;
			}
		}
	fprintf(f,"elements accessibe from col pointers: %d\n", i );

	fprintf(f,"row pointer array:\n");
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		if(e == NULL ) fprintf(f,"%d NULL\n", i);
		else fprintf(f,"%d (%d, %d)\n", i, e->i, e->j);
		}
	fprintf(f,"col pointer array:\n");
	for( j=0; j<m; j++ ) {
		e= A->col[j];
		if(e == NULL ) fprintf(f,"%d NULL\n", i);
		else fprintf(f,"%d (%d, %d)\n", j, e->i, e->j);
		}
	fprintf(f,"elements\n");
	for( i=0; i<n; i++ ) {
		e= A->row[i];
		while( e!=NULL ) {
			fprintf(f,"(%d, %d) value %f ", e->i, e->j, e->x);
			if( e->no == NULL ) fprintf(f,"no NULL ");
			else fprintf(f,"no (%d, %d) ", e->no->i, e->no->j );
			if( e->nd == NULL ) fprintf(f,"nd NULL\n");
			else fprintf(f,"nd (%d, %d)\n", e->nd->i, e->nd->j );
			e=e->no;
			}
		}
	return(0);
	}
	
	
	
	
	
/*solves G x = d by damped-least squares using bicongugate gradients, that is, by solving
  (G-transpose G + e2 I) x = G-transpose b for x , but not ever computing (G-transpose G + e2 I)
  directly.
   G: sparse matrix (must have rows >= columns)
   d: l.h.s. vector
   e2: damping parameter, must be non-negative
   x: solution; initialize to guess (e.g. 0.0) before call
   itol: 1, 2, 3, 4 depending on choice of convergence test
   tol: tolerance
   itmax: maximum iterations
   iter: actual iterations
   err: error estimate
   */
int sparse_dls_biconj( struct sparse_matrix *G, double *d, double *x, double e2, int itol, double tol,
	int itmax, int *iter, double *err, int *biec)
{
	int j, m, n, n_diagonal; // i,
	double ak, akden, bk, bkden, bknum, bnorm, dxnorm, xnorm, zm1norm, znorm, t;
	double *p=NULL, *pp=NULL, *r=NULL, *rr=NULL, *z=NULL, *zz=NULL, *b=NULL, *mtemp=NULL, *diagonal=NULL;
	

	if( G==NULL ) {
		fprintf(stderr,"error: biconj: NULL Matrix\n");
		*biec=1;
		return(0);
		}
	
	/* the reversal of the sense of m and n is intentional */
	m=G->n; /*rows*/
	n=G->m; /*columns*/

	if( (itol<1) || (itol>4) ) {
		fprintf(stderr,"error: biconj: bad itol\n");
		*biec=3;
		return(0);
		}
		
	diagonal=AT_times_A_pe2( G, e2, &n_diagonal);
	if( diagonal==NULL ) {
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}
	if( n_diagonal!=n ) {
		fprintf(stderr,"error: bicong: internal error: size mismatch\n");
		*biec=4;
		return(0);
		}
		
	b=(double*)malloc((n+10)*sizeof(double));
	mtemp=(double*)malloc((m+10)*sizeof(double));
	p=(double*)malloc((n+10)*sizeof(double));
	pp=(double*)malloc((n+10)*sizeof(double));
	r=(double*)malloc((n+10)*sizeof(double));
	rr=(double*)malloc((n+10)*sizeof(double));
	z=(double*)malloc((n+10)*sizeof(double));
	zz=(double*)malloc((n+10)*sizeof(double));
	if( (p==NULL) || (pp==NULL) || (r==NULL) || (rr==NULL)
         || (b==NULL) || (z==NULL) || (zz==NULL) || (mtemp==NULL) ) {
         	if( b!=NULL ) free(b);
		if( p!=NULL ) free(p);
         	if( mtemp!=NULL ) free(mtemp);
		if( pp!=NULL ) free(pp);
		if( r!=NULL ) free(r);
		if( rr!=NULL ) free(rr);
		if( z!=NULL ) free(z);
		if( zz!=NULL ) free(zz);
		fprintf(stderr,"error: bicong: out of memory\n");
		*biec=4;
		return(0);
		}
		
	smt_times_v( G, d, b );

	*iter=0;
	sparse_GTGpe2times(G,x,r,e2,mtemp);
	for( j=0; j<n; j++ ) {
		r[j]=b[j]-r[j];
		rr[j]=r[j];
		}
	znorm=1.0;
	if( itol==1) {
		bnorm=snorm(n,b,itol);
		}
	else if( itol==2 ) {
		sparse_GTGpe2solve(diagonal,b,z,n);
		bnorm=snorm(n,z,itol);
		}
	else {
		sparse_GTGpe2solve(diagonal,b,z,n);
		bnorm=snorm(n,z,itol);
		sparse_GTGpe2solve(diagonal,r,z,n);
		znorm=snorm(n,z,itol);
		}

	sparse_GTGpe2solve(diagonal,r,z,n);

	while( (*iter) <= itmax ) {
		(*iter)++;
		zm1norm=znorm;
		sparse_GTGpe2solve(diagonal,rr,zz,n);
		bknum=0.0;
		for( j=0; j<n; j++ ) bknum+=z[j]*rr[j];
		if( (*iter)==1 ) {
			for( j=0; j<n; j++ ) {
				p[j]=z[j];
				pp[j]=zz[j];
				}
			}
		else {
			bk=bknum/bkden;
			for( j=0; j<n; j++ ) {
				p[j]=bk*p[j]+z[j];
				pp[j]=bk*pp[j]+zz[j];
				}
			}
		bkden=bknum;
		sparse_GTGpe2times(G,p,z,e2,mtemp);
		akden=0.0;
		for( j=0; j<n; j++ ) akden+=z[j]*pp[j];
		ak=bknum/akden;
		sparse_GTGpe2times(G,pp,zz,e2,mtemp);
		for( j=0; j<n; j++ ) {
			x[j]+=ak*p[j];
			r[j]-=ak*z[j];
			rr[j]-=ak*zz[j];
			}
		sparse_GTGpe2solve(diagonal,r,z,n);
		if( (itol==1) || (itol==2) ) {
			znorm=1.0;
			(*err)=snorm(n,r,itol)/bnorm;
			}
		else {
			znorm=snorm(n,z,itol);
			if( (t=fabs(zm1norm-znorm)) > (EPS*znorm) ) {
				dxnorm=fabs(ak)*snorm(n,p,itol);
				(*err)=znorm/t*dxnorm;
				}
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			xnorm=snorm(n,x,itol);
			if( (*err)<=(0.5*xnorm) ) (*err)/=xnorm;
			else {
				(*err)=znorm/bnorm;
				continue;
				}
			}
		if( (*err)<=tol) break;
		}

	free(p); free(pp); free(r); free(rr); free(z); free(zz); free(b); free(mtemp); free(diagonal);
	if( (*err)<=tol) {*biec=0; return(1);}
	else {*biec=(-1); return(0);}
	}
	
/* internal routine, not intended as user callable */
/* performs r = ( G-transpose G + e2 I ) x */
int sparse_GTGpe2times( struct sparse_matrix *G, double *x, double *r, double e2, double *mtemp )
{
	int i, m;
	
	m = G->m;
	sm_times_v( G, x, mtemp );
	smt_times_v( G, mtemp, r );
	for( i=0; i<m; i++ ) r[i] += e2*x[i];
	return(1);
	}
	
/* internal routine, not intended as user callable */
int sparse_GTGpe2solve( double *diagonal, double *b, double *x, int n )
/* solves ( G-transpose G + e2 I ) x = b by ignoring all but main diagonal */
{
	int i;
	double t;
	for( i=0; i<n; i++ ) {
		t=diagonal[i];
		if( t!=0.0 ) x[i]=b[i]/t;
		else x[i]=0.0;
		}
	return(1);
	}
	
	
/* diagonal of A-transpose * A + e2 I*/
double  *AT_times_A_pe2( struct sparse_matrix *A, double e2, int *M)
{
	// struct sparse_matrix *C; //unused
	struct sparse_elem *eA, *eB;
	int i, j, kA, kB, mA,  nA,  nB, mB, nC; // iscolB,//unused
	double sum, *diagonal;

	if( A==NULL ) return(NULL);
	nA=A->n; mA=A->m;
	nB=A->n; mB=A->m;
	nC=mA;
	
	if( (diagonal=(double *)malloc(nC*sizeof(double))) == NULL ) {
		*M=0;
		return(NULL);
		}
	else  *M=nC;

	if( A->sym  ) {
		fprintf(stderr,"error: AT_times_A: sorry: case of symmetric A not implemented\n");
		exit(-1);
		}
	else {
		for( i=0; i<nC; i++ ) { 
			j=i;
			sum=0.0;
			eA=A->col[i]; eB=A->col[j];
			while( (eA!=NULL) && (eB!=NULL) ) {
				kA=eA->i; kB=eB->i;
				if( kA==kB ) sum += eA->x * eB->x;
				if( kA <= kB ) eA=eA->nd; else eB=eB->nd;
				}
			diagonal[i]=sum+e2;
			}
		}


	return(diagonal);
	}
	


